# Generated by Django 2.0 on 2018-06-21 22:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0002_auto_20180221_1439'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='emailtemplate',
            name='id',
        ),
        migrations.RemoveField(
            model_name='smstemplate',
            name='id',
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='slug',
            field=models.SlugField(editable=False, max_length=100, primary_key=True, serialize=False, verbose_name='slug'),
        ),
        migrations.AlterField(
            model_name='smstemplate',
            name='slug',
            field=models.SlugField(editable=False, max_length=100, primary_key=True, serialize=False, verbose_name='slug'),
        ),
    ]
